
/*
 * Copyright (c) 2011, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <ecto/ecto.hpp>
#include <ecto/registry.hpp>

using ecto::tendrils;
namespace ecto_test
{
  struct ParameterWatcher
  {
    double value_;

    static void declare_params(ecto::tendrils& p)
    {
      p.declare<double> ("value", "I use this value", 1.0).required(true);
    }

    static void declare_io(const ecto::tendrils& parameters, ecto::tendrils& inputs, ecto::tendrils& outputs)
    {
      inputs.declare<double> ("input", "input");
      outputs.declare<double> ("output", "output");
      outputs.declare<double> ("value", "the parameter.");
    }

    void onvalue_change(double v)
    {
      std::cout << "old value: " << value_ << std::endl;
      std::cout << "new value: " << v << std::endl;
      value_ = v;
    }

    void configure(const tendrils& parms, const tendrils& inputs, const tendrils& outputs)
    {
      parms["value"]->set_callback<double>(boost::bind(&ParameterWatcher::onvalue_change, this, _1));
    }

    int process(const ecto::tendrils& inputs, const ecto::tendrils& outputs)
    {
      double in;
      inputs["input"] >> in;
      outputs["output"] << in * value_;
      outputs["value"] << value_;
      return ecto::OK;
    }
  };
}

ECTO_CELL(ecto_test, ecto_test::ParameterWatcher, "ParameterWatcher", "Uses parameter change callbacks.");

